unit uFenster;
(* ******************************************************************** *)
(* K L A S S E : TFensterFrm                                            *)
(* -------------------------------------------------------------------- *)
(* Version     : 3.0                                                    *)
(* Autor       : (c) S. Spolwig, OSZ Handel, 10997 Berlin, 1999-2006    *)
(*                                                                      *)
(* Aufgabe     : FensterFrm vereint alle Komponenten der Bedienungs-    *)
(*               oberflaeche des Geldwechslers und steuert den Ablauf   *)
(*               von der Geldannahme bis zur Ausgabe des Wechselgeldes  *)
(*               Es nimmt alle Eingaben des Benutzers entgegen und zeigt*)
(*               die jeweiligen Daten als Objekte auf dem Bildschirm an.*)
(*                                                                      *)
(* Compiler    : Delphi 4.0                                             *)
(* Aenderung   : V. 2.03  -  10-JAN-2000  kleine bugs beseitigt         *)
(*                  3.0   -  09-FEB-2006  EURO                          *)
(* ******************************************************************** *)

INTERFACE
(* ==================================================================== *)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, ComCtrls,
  uZeit,
  uGrafik,

  uNote,
  uFuenfEuro,
  uZehnEuro,
  uWechselEinheit;

type
  TFensterFrm = class(TForm)
    AusgabeklappePnl: TPanel;
    NoteneinzugEdt  : TEdit;
    AusgabeImg      : TImage;
    EinzugImg       : TImage;
    DisplayPnl      : TPanel;
    DisplayEdt      : TEdit;
    BAnleitungMmo   : TMemo;

    procedure FormCreate (Sender: TObject);
    procedure NoteneinzugEdtKeyPress (Sender: TObject; var Key: Char);
    procedure AusgabeImgClick (Sender: TObject);
    procedure EinzugImgClick (Sender: TObject);
    procedure FormClose (Sender: TObject; var Action: TCloseAction);

   private
    procedure AusgabebildInit;
    procedure EinzugbildInit;
    procedure Reset;
    procedure NoteEinziehenPruefen(Note : TNote);
    procedure MuenzenEntnehmen;
    procedure BankNoteEntnehmen;
  end; (* class *)

var
  FensterFrm     : TFensterFrm;
  WechselEinheit : TWechselEinheit;

IMPLEMENTATION
(* ==================================================================== *)
{$R *.DFM}

var
	Zeichenblatt : TZeichenblatt;

procedure TFensterFrm.FormCreate(Sender: TObject);
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Grafik initialisieren; aktuelle Exemplare erzeugen         *)
(*  vorher : -                                                          *)
(*  nachher:                                                            *)
(* -------------------------------------------------------------------- *)
begin
  Zeichenblatt       := TZeichenblatt.Create; // fuer die graf. Darstellungen
  WechselEinheit     := TWechselEinheit.Create;
  WechselEinheit.SchaechteFuellen;
  AusgabebildInit;
  AusgabeImg.Enabled := false;  // damit nix aus dem leeren Schacht entnommen wird
  Reset;
end;


procedure TFensterFrm.NoteneinzugEdtKeyPress(Sender: TObject; var Key: Char);
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Geldtaste des Benutzer annehmen; entsprechendes            *)
(*           Muenzexemplar erzeugen und der WechselEinheit uebergeben   *)
(*  vorher : -                                                          *)
(*  nachher: Muenzschlitz ist leer                                      *)
(* -------------------------------------------------------------------- *)
var
  ZehnEuro   : TZehnEuro;
  FuenfEuro: TFuenfEuro;
  Falschgeld : TNote;

begin
  EinzugbildInit;
  NoteneinzugEdt.Enabled := false; // damit nix doppelt eingegeben wird
  case Key of

      'f' : begin
              FuenfEuro := TFuenfEuro.Create;
              NoteEinziehenPruefen(FuenfEuro);
              AusgabeImg.Enabled := true;      // damit Muenzen angezeigt werden
          end;

      'z' : begin
              ZehnEuro := TZehnEuro.Create;
              NoteEinziehenPruefen(ZehnEuro);
              AusgabeImg.Enabled := true;          
            end;
        else
            begin
              Falschgeld := TNote.Create;
              Falschgeld.SetPos(2,20,130,262);
              Falschgeld.SetPraegung('.\bilder\10-EU-falsch.bmp');
              NoteEinziehenPruefen(Falschgeld);
              AusgabeImg.Enabled := false;
            end;
   end; // case
end;


procedure TFensterFrm.NoteEinziehenPruefen(Note : TNote);
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Note einziehen und Echtheit pruefen. Wenn echt, dann       *)
(*           annehmen und Muenzen auswerfen. Falschgeld wieder aus-     *)
(*           werfen.                                                    *)
(* -------------------------------------------------------------------- *)
begin
   EinzugbildInit;
   Note.zeigen;
   Note.BewegenNach(0,-1);
   Note.Loeschen;

   if (Note is TZehnEuro) AND (note.GetFarbe = RGB(198,94,117))  OR  // Echtheit
      (Note is TFuenfEuro) AND (note.GetFarbe = RGB(121,135,171))    // pruefen
   then
     begin
       if WechselEinheit.HatZuwenigMuenzen
       then
         begin
          DisplayEdt.Text := ' S T  R U N G ';
          WechselEinheit.NoteAuswerfen(Note);
         end
       else
          begin
            WechselEinheit.Geldannehmen(Note);
            AusgabebildInit;
            WechselEinheit.MuenzenAuswerfen;
            DisplayEdt.Text := ' Bitte Geld entnehmen ';
          end;
     end
   else WechselEinheit.NoteAuswerfen(Note);   // nicht echt
end;


procedure TFensterFrm.AusgabeImgClick(Sender: TObject);
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Muenzen aus der Ausgabeklappe entnehmen, neue Eingabe      *)
(*           vorbereiten                                                *)
(*  vorher : Objekte exisitieren                                        *)
(*  nachher: Ausgabeklappe ist leer.                                    *)
(* -------------------------------------------------------------------- *)
begin
  MuenzenEntnehmen;
  Reset;
  NoteneinzugEdt.SetFocus;
end;


procedure TFensterFrm.FormClose(Sender: TObject; var Action: TCloseAction);
(* -------------------------------------------------------------------- *)
(* Aufgabe: Aufrauemen, Hinweis zeigen, Fenster schlieen,              *)
(*           Programm beenden                                           *)
(* -------------------------------------------------------------------- *)
begin
  DisplayEdt.Color := clRED;
  DisplayEdt.Text  := ' Ausser Betrieb ';
  BAnleitungMmo.lines.LoadfromFile('Wartung.txt');
  Zeit.Setpause(2000);
end;


procedure TFensterFrm.EinzugImgClick(Sender: TObject);
(* -------------------------------------------------------------------- *)
(*  Aufgabe: BanknoteEntnehmen                                          *)
(* -------------------------------------------------------------------- *)
begin
  BanknoteEntnehmen;
  NoteneinzugEdt.Enabled := true;
  NoteneinzugEdt.SetFocus;
end;


procedure TFensterFrm.AusgabebildInit;
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Grafik initialisieren fr Ausgabeschacht, leer zeigen      *)
(* -------------------------------------------------------------------- *)
begin
  Zeichenblatt.SetLink(AusgabeImg);
  Zeichenblatt.SetFuellfarbe(clgray);
  Zeichenblatt.Setfarbe(clgray);
  Zeichenblatt.Zeigen;
end;


procedure TFensterFrm.EinzugbildInit;
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Grafik initialisieren fr Geldeinzug, leer zeigen          *)
(* -------------------------------------------------------------------- *)
begin
  Zeichenblatt.SetLink(EinzugImg);
  Zeichenblatt.Setfarbe(clBtnFace);
  Zeichenblatt.SetFuellfarbe(clBtnFace);
  Zeichenblatt.Zeigen;
end;


procedure TFensterFrm.MuenzenEntnehmen;
(* -------------------------------------------------------------------- *)
(* Aufgabe: Ausgabeklappe leeren                                        *)
(* -------------------------------------------------------------------- *)
begin
  AusgabebildInit;
end;


procedure TFensterFrm.BanknoteEntnehmen;
(* -------------------------------------------------------------------- *)
(* Aufgabe: Falschgeld und zurckgegebene Noten vom Bildschirm loeschen *)
(* -------------------------------------------------------------------- *)
begin
  EinzugbildInit;
end;


procedure TFensterFrm.Reset;
(* -------------------------------------------------------------------- *)
(*  Aufgabe: Automat auf neue Eingabe vorbereiten: Einwurf oeffnen      *)
(*           Eingabeaufforderung                                        *)
(* -------------------------------------------------------------------- *)
begin
  NoteneinzugEdt.Text    := '';
  NoteneinzugEdt.Enabled := true;
  DisplayEdt.Text        := ' Bitte Geld einfdeln ';
end;




END.
