UNIT uQListe;
(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E   : TQListe - abstrakte Klasse                           *)
(* -------------------------------------------------------------------- *)
(* Version       : 1.0                                                  *)
(* Autor         : (c) S. Spolwig, OSZ-Handel I, 10997 Berlin           *)
(*                                                                      *)
(* Aufgabe       : Allgemeine statische Liste (Queue)zur Verwaltung     *)
(*                 beliebiger Objekte. Hinzufuegen eines Elements am    *)
(*                 Ende (Append), Entnehmen (GetElement) am Anfang.     *)
(*                 In abgeleiteten Klassen sollte der Elemententyp de-  *)
(*                 klariert und die abstrakten Methoden Load, Store     *)
(*                 ueberschrieben werden.                               *)
(* Compiler      : Delphi 4.0                                           *)
(* Aenderung     : V. 1.0   - 20-DEC-99                                 *)
(* ******************************************************************** *)

INTERFACE
(* ==================================================================== *)
const
   MAXLAENGE = 10;

type
  TQListe  = class (TObject)
             protected
              Kollektion  : array [0..MAXLAENGE + 1] of Pointer;
              LiLaenge    : Word;     // Anzahl der belegten Elemente

             public
              constructor Create;                      virtual;
              procedure Init;                          virtual;

              procedure Append (Elem : pointer);       virtual;
              function  GetElement: pointer;           virtual;

              procedure RemoveAll;                     virtual;
              function  GetLen  : integer ;            virtual;
              function  IsEmpty : boolean;             virtual;
              function  IsFull  : boolean;             virtual;

              procedure Load (Dateiname : string);     virtual;
              procedure Store(Dateiname : string);     virtual;
            end;

(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TObject
Bezugsklassen  : -


Methoden
--------

Create
  Auftrag : Leere L. erzeugen
  vorher  : -
  nachher : Lilaenge ist Null.

Init
  Auftrag : L. initial.
  vorher  : L. ist vorhanden
  nachher : Lilaenge ist Null


GetElement
  Anfrage : Zugriff auf das erste Element zum Lesen
  vorher  : Die Liste ist init.
  nachher : Liefert das aktuelle Element. Danach ist es aus der Liste entfernt.
            Wenn die Liste leer war wird NIL geliefert.

Append
  Auftrag : Neues Element an das Ende der Liste anhaengen.
            Wenn die Liste leer war, ist das neue El. das erste.
  vorher  : Die Liste ist initialisiert
  nachher : Listenlaenge ist um eins erhoeht. Ist die Liste voll,
            geschieht nichts.


RemoveAll
  Auftrag: Alle Elemente aus der Liste entfernen
  vorher : -
  nachher: Die Liste ist leer.

IsEmpty
  Anfrage: ob Liste leer ist
  vorher : Die Liste ist initialisiert.
  nachher: True, wenn die Liste leer ist.

IsFull
  Anfrage: ob Liste voll ist
  vorher : Die Liste ist initialisiert.
  nachher: True, wenn LiLaenge = MAXLISTE

GetLen
  Anfrage: gibt Listenlaenge zurueck
  vorher : Liste ist initialisiert
  nachher: -


Load
  Auftrag: Liste aus externer Datei laden.
  vorher : nicht definiert
  nachher: nicht definiert.


Store
  Auftrag: Liste in externe Datei speichern
  vorher : nicht definiert.
  nachher: nicht definiert.

----------------------------------------------------------------------- *)


IMPLEMENTATION
(* ==================================================================== *)

constructor TQListe.Create;
(* -------------------------------------------------------------------- *)
begin
  inherited Create;
  Init;
end;


procedure TQListe.Init;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  for i := 0 to MAXLAENGE +1 do
  begin
    Kollektion[i] := NIL;
  end;
  Lilaenge    := 0;
end;


function TQListe.GetElement : pointer;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  Result := NIL;

  if NOT IsEmpty
  then
    begin
      Result := Kollektion[1];
      if LiLaenge > 1
      then
      for i := 2 to Lilaenge do
      begin
        Kollektion[i-1] := Kollektion[i];
      end;
      dec(Lilaenge);
      Kollektion[Lilaenge + 1] := NIL;
    end
end;


procedure TQListe.Append (Elem : pointer);
(* -------------------------------------------------------------------- *)
begin
  if NOT IsFull
  then
    begin
      Kollektion[LiLaenge + 1 ] := Elem;
      inc (Lilaenge);
    end;
end;


procedure TQListe.RemoveAll;
(* -------------------------------------------------------------------- *)
begin
  if NOT IsEmpty
  then Init;
end;  (* remove *)


function TQListe.GetLen : integer ;
(* -------------------------------------------------------------------- *)
begin
  Result := Lilaenge;
end;


function TQListe.IsEmpty : boolean ;
(* -------------------------------------------------------------------- *)
begin
  Result := false;
  if LiLaenge = 0
  then Result := true;
end;


function TQListe.IsFull : boolean ;
(* -------------------------------------------------------------------- *)
begin
  Result := false;
  if LiLaenge = MAXLAENGE
  then Result := true;
end;



procedure TQListe.Load (Dateiname : string);
(* -------------------------------------------------------------------- *)
begin
end;


procedure TQListe.Store(Dateiname : string);
(* -------------------------------------------------------------------- *)
begin
end;

END.
