UNIT uWechselEinheit;
(* ******************************************************************** *)
(* K L A S S E : TWechselEinheit                                        *)
(* -------------------------------------------------------------------- *)
(* Version     : 3.x                                                    *)
(* Autor       : (c) S. Spolwig, OSZ Handel I, 10997 Berlin             *)
(*                                                                      *)
(* Aufgabe     : Stellt das Bauteil zur Annahme und Ausgabe von Noten   *)
(*               und Muenzen dar. Es steuert den internen Wechselvorgang*)
(*               Die Rueckgabe erfolgt nur in Muenzen. Falschgeld wird  *)
(*               wieder ausgeworfen.                                    *)
(*               Die auszugebenden Muenzen werden einem Ausgabeschacht  *)
(*               uebergeben. Keine kaufm. Buchfuehrung.                 *)
(*                                                                      *)
(* Compiler    : Delphi 6.0                                             *)
(* Aenderung   : V. 2.031  - 16-JAN-2000  - alle Muenzen                *)
(*                  3.0    - 11-FEB-2006                                *)
(* ******************************************************************** *)
INTERFACE
(* ==================================================================== *)

USES
   SysUtils,
   uZeit,
   uGrafik,
   uQListe,  // fuer Schchte

   uZehnEuro,
   uFuenfEuro,
   uZweiEuro,
   uEinEuro,
   uMuenze,
   uFuenfziger,
   uZwanziger,
   uZehner,
   uNote;

type
   TWechselEinheit = class (TObject)
      private
       Schacht10Eu,
       Schacht5Eu,
       Schacht2Eu,
       Schacht1Eu,
       Schacht50,
       Schacht20,
       Schacht10,
       Ausgabeschacht : TQListe;
      public
       constructor Create;
       procedure SchaechteFuellen;
       procedure GeldAnnehmen (Note : TNote);
       procedure ZehnEuroWechseln;
       procedure FuenfEuroWechseln;
       procedure MuenzenAuswerfen;
       procedure NoteAuswerfen (Note : TNote);
       function  HatZuwenigMuenzen : boolean;
   end;

(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : -
Bezugsklassen  : TZeit, TGrafik, TQListe, alle Muenz-Klassen, SysUtils

Methoden
--------
Create
  Auftrag : WechselEinheit und mit Schaechten erzeugen
  vorher  : -
  nachher : Schaechte sind leer.


SchaechteFuellen
  Auftrag : Muenzschaechte mit entsprechenden Muenzen fuellen
  vorher  : -
  nachher : Schaechte sind voll.


GeldAnnehmen
  Aufgabe: Eingeworfene (eingetippte) Sorte uebernehmen und Wechselmethode
           starten
  vorher : -
  nachher: -

ZehnEuroWechseln
  Aufgabe : 2-EU, 1-EU, 50-C erzeugen und an Ausgabeschacht uebergeben
  vorher  : -
  nachher : Die Muenzen sind im Ausgabeschacht

FuenfEuroWechseln
  Aufgabe : 2-EU, 1-EU, 50-C. 20-C, 10-C erzeugen und an Ausgabeschacht uebergeben
  vorher  : -
  nachher : Die Muenzen sind im Ausgabeschacht

MuenzenAuswerfen
  Auftrag: Muenzen aus dem Ausgabeschacht holen und zeigen
  vorher : Ausgabeschacht ist nicht leer
  nachher: Ausgabeschacht ist leer

NoteAuswerfen
  Auftrag: Banknote wieder zurck.
  vorher : Note wurde angenommen
  nachher: -

HatZuwenigMuenzen : boolean
  Anfrage: ob in den Schaechten noch gengend Muenzen zur nchsten Ausgabe
           vorhanden sind.
           Wahr, wenn Anzahl in einem der Schchte zu gering.
  vorher : -
  nachher: -
----------------------------------------------------------------------- *)

IMPLEMENTATION
(* ==================================================================== *)

var
  ZweiEuro  : TZweiEuro;
  EinEuro   : TEinEuro;
  Fuenfziger: TFuenfziger;
  Zwanziger : TZwanziger;
  Zehner    : TZehner;

constructor TWechselEinheit.Create;
(* -------------------------------------------------------------------- *)
begin
  inherited Create;
  Ausgabeschacht := TQListe.Create;
  Schacht10Eu    := TQListe.Create;
  Schacht5Eu     := TQListe.Create;
  Schacht2Eu     := TQListe.Create;
  Schacht1Eu     := TQListe.Create;
  Schacht50      := TQListe.Create;
  Schacht20      := TQListe.Create;
  Schacht10      := TQListe.Create;
end;


procedure TWechselEinheit.SchaechteFuellen;
(* -------------------------------------------------------------------- *)
begin
  while NOT Schacht2Eu.IsFull do    // alle werden gleichmaessig gefuellt
  begin
    ZweiEuro := TZweiEuro.Create;
    Schacht2Eu.Append(ZweiEuro);

    EinEuro := TEinEuro.Create;
    Schacht1Eu.Append(EinEuro);

    Fuenfziger := TFuenfziger.Create;
    Schacht50.Append(Fuenfziger);

    Zwanziger := TZwanziger.Create;
    Schacht20.Append(Zwanziger);

    Zehner := TZehner.Create;
    Schacht10.Append(Zehner);
  end;
end;


procedure TWechselEinheit.GeldAnnehmen (Note : TNote);
(* -------------------------------------------------------------------- *)
begin
  Beep;
  if Note is TZehnEuro
  then
    begin
      Schacht10Eu.Append(Note);
      ZehnEuroWechseln;
    end;

  if Note is TFuenfEuro
  then
    begin
      Schacht5Eu.Append(Note);
      FuenfEuroWechseln;
    end;
end;


procedure TWechselEinheit.ZehnEuroWechseln;
(* -------------------------------------------------------------------- *)
(* Auftrag: Muenzen  aus dem Vorratsschacht in den Ausgabeschacht tun   *)
(*          Wenn nicht genug vorhanden,  geschieht nichts               *)
(* -------------------------------------------------------------------- *)
begin
  Ausgabeschacht.Init;

  if NOT HatZuwenigMuenzen
  then
    begin
      Ausgabeschacht.Append(Schacht2Eu.GetElement);
      Ausgabeschacht.Append(Schacht2Eu.GetElement);
      Ausgabeschacht.Append(Schacht2Eu.GetElement);
      Ausgabeschacht.Append(Schacht1Eu.GetElement);
      Ausgabeschacht.Append(Schacht1Eu.GetElement);
      Ausgabeschacht.Append(Schacht1Eu.GetElement);
      Ausgabeschacht.Append(Schacht50.GetElement);
      Ausgabeschacht.Append(Schacht50.GetElement);
    end;
end;


procedure TWechselEinheit.FuenfEuroWechseln;
(* -------------------------------------------------------------------- *)
(* Auftrag: Muenzen  aus dem Vorratsschacht in den Ausgabeschacht tun   *)
(*          Wenn nicht genug vorhanden, geschieht nichts                *)
(* -------------------------------------------------------------------- *)
begin
  Ausgabeschacht.Init;

  if NOT HatZuwenigMuenzen
  then
    begin
      Ausgabeschacht.Append(Schacht2Eu.GetElement);
      Ausgabeschacht.Append(Schacht1Eu.GetElement);
      Ausgabeschacht.Append(Schacht50.GetElement);
      Ausgabeschacht.Append(Schacht20.GetElement);
      Ausgabeschacht.Append(Schacht1Eu.GetElement);
      Ausgabeschacht.Append(Schacht10.GetElement);
      Ausgabeschacht.Append(Schacht10.GetElement);
    end;
end;


procedure TWechselEinheit.MuenzenAuswerfen;
(* -------------------------------------------------------------------- *)
(* Muenzen aus dem Ausgabeschacht holen, Position berechnen und zeigen  *)
(* -------------------------------------------------------------------- *)
const
  XStart =  5;  // wo die 1. hinfaellt
  YStart =  50;
var
  abstand  : integer;
  Muenze   : TMuenze;

begin
  abstand := 0;

  while NOT Ausgabeschacht.IsEmpty do
  begin
    Muenze := Ausgabeschacht.GetElement;
    if muenze is TZweiEuro
    then
      begin
        Muenze.SetPos(XStart + abstand, YStart - 3, XStart + 65, YStart + 77);
        abstand := abstand + 72;
      end
    else
      if muenze is TEinEuro
      then
        begin
          Muenze.SetPos(XStart + abstand, YStart,XStart + 63, YStart + 77 );
          abstand := abstand + 68;
        end
     else
       if muenze is TFuenfziger
       then
         begin
          Muenze.SetPos(XStart + abstand, YStart, XStart + 65, YStart + 77 );
          abstand := abstand + 68;
        end
      else  // fr die restlichen
        begin
          Muenze.SetPos(XStart + abstand, YStart + 15, XStart + 52, YStart + 5  );
          abstand := abstand + 60;
        end;

    Muenze.Zeigen;

    Beep;
    Zeit.SetPause(200);
  end; // while
end;


procedure TWechselEinheit.NoteAuswerfen (Note : TNote) ;
(*-------------------------------------------------------------------- *)
begin
  Note.BewegenNach(0,1);   // unten
  Beep;
end;


function TWechselEinheit.HatZuwenigMuenzen : boolean;
(*-------------------------------------------------------------------- *)
begin
  Result := false;
  if
   (Schacht2Eu.GetLen < 3) or
   (Schacht1Eu.GetLen < 3) or
   (Schacht50.GetLen  < 2) or
   (Schacht20.GetLen  < 1) or
   (Schacht10.GetLen  < 3)
  then Result := true;
end;

END.
